package SM.diglib;

import org.w3c.rdf.*;
import org.ginf.*;
import org.ginf.helpers.*;
import org.ginf.delivery.HTTP.*;
import org.ginf.delivery.*;
import org.ginf.schemas.*;

import java.util.*;
import java.io.*;

public class Client implements Layer {

  RDFFactory factory;
  Layer outside;
  int limit;

  public Client() throws Exception {
    factory = new RDFFactoryImpl();
    SchemaRegistryImpl reg = (SchemaRegistryImpl)factory.registry();

    reg.addLoader( new DublinCore() ); // do not attempt to download DublinCore schema
    reg.addLoader( new Coll() );
    reg.addLoader( new SDLIP() );
      
  }

  /**
   * Layer implementation
   */
  public void connect(String what, Layer layer) {
    if("transport".equals(what))
      this.outside = layer;
    else
      throw new IllegalArgumentException("[Client] cannot connect \"" + what + "\"");
  }

  public Model dispatch(Model model, Layer from) throws Exception {

    SchemaModel m = factory.createSchemaModel( model ); // we want to use schema info
    // Validate schema conformance
    //    System.err.println("GOT MODEL");
    //    DefaultModel.printTriples( m, System.err );
    m.validate();
    // check whether is Result
    //    Triple t = m.find1( null, RDF.type, SDLIP.Result );
    //    if( t != null ) {
    Resource res = null;

    try {
      res = m.find1( null, RDF.type, SDLIP.Result ).subject();
    } catch (Exception any) {}
      // print some statistics
    try {
      //      Resource request = (Resource)m.find1( res, CoreComm.from, null ).object();
      System.err.println("GOT RESULT FOR: " + m.find1( res, CoreComm.from, null ).object());
    } catch (Exception any) {}
      //      Resource state = m.find1( null, RDF.type, HTTP.Connection ).subject();
      //      System.err.println("GOT RESULT VIA: " + state + " FROM " +
      //			 m.find1( state, HTTP.url, null ).object());
      //			 m.find1( state, HTTP.host, null ).object() + ":" +
      //			 m.find1( state, HTTP.port, null ).object());
      // process result
    receiveReply( res, m );
    //    }
    // wake up
    synchronized(this) { notify(); }
    return null;
  }

  Resource getStateInfoClass() {
    return HTTP.Connection;
  }

  void hangup(Model m) {
    // hang up
    try {
      Resource state = m.find1( null, RDF.type, getStateInfoClass()).subject();
      System.err.println("STATE: " + state);
      Model hangup = factory.createModel();
      Resource msg = RDFUtil.noname();
      hangup.add( msg, RDF.type, CoreState.DiscardState );
      hangup.add( msg, CoreState.state, state );
      hangup.add( state, RDF.type, CoreState.StateInfo );
      outside.dispatch( hangup, this );
    } catch (Exception any) {
      System.err.println("Could not hang up: " + any);
    }
  }

  //  Thread thread;

  void receiveReply(Resource res, Model m) throws Exception {

    //    factory.createRDFMS().serialize( m, new OutputStreamWriter(System.err) );

    // print result size
    try {
      System.out.println("Result size is: " + m.find1( res, SDLIP.resultSize, null ).object());
    } catch (Exception any) {}

    // print out results
    Model items = m.find( res, Coll.item, null );

    System.err.println("FOUND " + items.size() /* + " for " + res*/);
    limit -= items.size();
    int count = 0;
    for(Enumeration en_i = items.elements(); en_i.hasMoreElements(); ) {
      Resource item = (Resource)((Triple)en_i.nextElement()).object();
      // print title and author
      try {
	System.out.println("Title is: " + m.find1( item, DublinCore.Title, null ).object());
	System.out.println("Author is: " + m.find1( item, DublinCore.Creator, null ).object());
      } catch (Exception any) {}
    }
    if(limit <= 0)
      hangup(m);
    
  }

  static final Resource myColl = new Resource("http://www-diglib.stanford.edu/diglib/CS-TR");

  static final int TIMEOUT = 60 * 1000; // 1 min

  void sendRequest(String url, String query, String limit) throws Exception {

    // we construct the model using the API, so fetch the schema manually
    factory.registry().fetch(SDLIP.NS);

    Model m = factory.createModel();
    
    // create new unique unnamed resource
    Resource s = RDFUtil.noname();
    m.add( s, RDF.type, SDLIP.Search ); // this is a Search request
    m.add( s, SDLIP.queryString, query ); // the query
    m.add( s, SDLIP.limit, limit ); // max 10 results
    m.add( s, SDLIP.scope, myColl );

    Resource attrs = RDFUtil.noname();
    m.add( s, SDLIP.attributes, attrs); // attributes
    m.add( attrs, RDF.type, RDF.Bag ); // this is a Bag of attribute names
    m.add( attrs, RDFUtil.ord(1), DublinCore.Title ); // 1=Title
    m.add( attrs, RDFUtil.ord(2), DublinCore.Creator ); // 2=Creator

    //    m.add( s, HTTP.host, "sole"); // where to send
    //    m.add( s, HTTP.invocationType, HTTP.Synch ); // wait for reply

    Resource dest = new Resource( url );
    m.add( s, CoreComm.to, dest );
    m.add( dest, RDF.type, CoreComm.ProcessingEntity);

    this.limit = Integer.parseInt(limit);


    //    System.err.println("SENDING MODEL");
    //    factory.createRDFMS().serialize( m, new OutputStreamWriter(System.err) );

    outside.dispatch(m, this);
  }

  void loopRequest(String url, String query, String limit) {

    long first = System.currentTimeMillis();
    try {

      int N = 1;
      
      do {
	
	long start = System.currentTimeMillis();
	//	  SchemaModel res = factory.createSchemaModel( outside.dispatch(m, this) );
	sendRequest(url, query, limit);
	// wait for the result to arrive
	//	  System.err.println("GO TO SLEEP...");
	
	try {
	  synchronized(this) { wait(TIMEOUT); }
	} catch (InterruptedException inex) {}
	  
	//	  System.err.println("AWAKEN!");
	//	  closeConnection(res);
	// process the reply
	//	  receiveReply(res);

	long end = System.currentTimeMillis();
	System.err.println("It took " + (end-start) + "ms to do the request. " +
			   "Average: " + (end - first) / N + "ms");
	N++;
      } while(true);
      
    } catch (Exception any) {
      System.err.println("Annotation client: " + any);
      any.printStackTrace(System.err);
    }
  }

  public void closeConnection(Model res) throws Exception {
    
    // close the connection
    /*
    Triple t_state = res.find1( null, RDF.type, HTTP.Connection );
    if(t_state != null) {
      System.err.println("Closing connection " + t_state.subject() );
      
      Model cl = factory.createModel();
      Resource cm = RDFUtil.noname();
      cl.add( cm, RDF.type, HTTP.DiscardState );
      cl.add( cm, HTTP.deliveryInfo, t_state.subject() );
      cl.add( t_state.subject(), RDF.type, HTTP.Connection );
      outside.dispatch(cl, this);
    }
    */
  }

  public String toString() {
    return "SDLIP Client Layer";
  }

  public static void main(String[] args) throws Exception {

    Client client = new Client();

    HTTPLayer h = new HTTPLayer(client.factory);
    //    ConnectionlessTransport t = new ConnectionlessTransport(client.factory, true);
    client.connect( "transport", h );
    //    t.connect( "service", client );
    //    t.connect( "http", h);

    //        client.loopRequest("http://sole.stanford.edu:8271/dlserver", "andreas", "2");
    client.sendRequest(args[0], "web", "4");
  }
}
