package SM.ginf;

import org.w3c.rdf.*;
import org.ginf.*;
import org.ginf.helpers.*;
import org.ginf.delivery.HTTP.*;
import org.ginf.delivery.*;
import org.ginf.schemas.*;

import java.util.*;
import java.io.*;

public class HelloWorldClient implements Layer {

  static final String NS = "http://www-diglib.stanford.edu/diglib/ginf/1999/05/26-hello-world#";
  static final Resource HelloWorld_GetString = new Resource(NS + "GetString");
  static final Resource HelloWorld_StringToPrint = new Resource(NS + "StringToPrint");
  static final Resource HelloWorld_string = new Resource(NS + "string");

  RDFFactory factory;
  Layer transport;

  public HelloWorldClient(RDFFactory factory) throws Exception {

    this.factory = factory;
  }

  public void connect(String what, Layer layer) {
    if("transport".equals(what))
      this.transport = layer;
    else
      throw new IllegalArgumentException(toString() + " cannot connect \"" + what + "\"");
  }

  /**
   * Layer implementation
   */

  public Model dispatch(Model model, Layer from) throws Exception {

    // we want to access schema information
    SchemaModel m = factory.createSchemaModel( model );

    try {
      // Optionally validate schema conformance
      m.validate();

      // find the reply
      Resource reply = m.find1( null, RDF.type, CoreComm.Message ).subject();
      
      // we can process only StringToPrint repplies
      if(m.find( reply, RDF.type, HelloWorld_StringToPrint ).size() > 0) {
	
	System.out.println( m.find1( reply, HelloWorld_string, null ).object().toString() );
      }
    } catch (Exception any) {
      System.err.println(toString() + " ERROR " + any.getMessage());
    }
    // done
    return null;
  }

  void sendRequest(String url) throws Exception {

    // we construct the model using the API, so fetch the schema manually
    factory.registry().fetch(NS);

    Model m = factory.createModel();
    
    // create new unique unnamed resource
    Resource s = RDFUtil.noname();
    // this is our request
    m.add( s, RDF.type, HelloWorld_GetString );
    // destination
    m.add( s, CoreComm.to, new Resource(url));
    m.add( new Resource(url), RDF.type, CoreComm.ProcessingEntity );

    transport.dispatch(m, this);
  }

  public String toString() {
    return "Hello World Client";
  }

  public static void main(String[] args) throws Exception {

    // create the default factory
    RDFFactory factory = new RDFFactoryImpl();
    
    // create the client
    HelloWorldClient client = new HelloWorldClient(factory);
    // create the transport layer
    HTTPLayer h = new HTTPLayer(factory);
    client.connect( "transport", h );
    
    client.sendRequest("http://localhost");
  }
}
