/**
 * Copyright  Sergey Melnik (Stanford University, Database Group)
 *
 * All Rights Reserved.
 */

package SM.ginf;

import org.ginf.*;
import org.ginf.helpers.*;
import org.ginf.schemas.*;
import org.xml.sax.*;

import java.net.*;
import java.io.*;

/**
 * This test parser reads and parses an RDF document located
 * at the specified URL and outputs the model as triples
 * and in a serialized form.
 */
public class TestParser {
  
  public static void main(String[] args) {
    
    if (args.length == 0) {
      System.err.println("Usage: java [-Dorg.xml.sax.parser=<classname>] SM.ginf.TestParser [ URI | filename ]");
      System.exit(1);
    }
    
    try {
      
      URL url = null;
      try {
	url = new URL (args[0]);
      } catch (Exception e) {
	url = new URL ("file", null, args[0]);
      }
      
      RDFFactory factory = new RDFFactoryImpl();
      RDFMS parser = factory.createRDFMS();

      // Prepare input source
      InputSource source = new InputSource( url.openStream() );
      source.setSystemId( url.toString() );

      Model m = parser.parse( source );

      System.err.println("\nMODEL AS TRIPLES:");
      RDFUtil.printTriples(m, System.err);

      System.err.println("\nMODEL AS XML:");
      parser.serialize( m, new OutputStreamWriter(System.out));

      System.err.print("\nVALIDATION: ");
      SchemaModel sm = factory.createSchemaModel( m );
      try {
	sm.validate();
	System.err.println("PASSED");
      } catch (Exception any) {
	System.err.println("FAILED: " + any.getMessage());
      }

    } catch (Exception any) {
      System.err.println("Could not parse/serialize: " + any);
      any.printStackTrace(System.err);
    }

  }
}
